// to do on future release
// -> look at response
// -> rate limiting
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as logger from 'logger';
import exceptionalError from 'exceptionalhandler';
import util from 'util';
import * as dotenv from 'dotenv';
import * as fdConstants from './constants.js';
dotenv.config();
const isProduction = (process.env.NODE_ENV === 'PRODUCTION');
let fdKey;
let fdPw;
if (isProduction) {
    fdKey = process.env.PROD_API_KEY;
    fdPw = process.env.PRO_API_PW;
}
else {
    fdKey = process.env.DEV_API_KEY;
    fdPw = process.env.DEV_API_PW;
}
const fdAuth = `Basic ${Buffer.from(`${fdKey}:${fdPw}`).toString('base64')}`;
export function performApiReq(endPoint, method, body) {
    return __awaiter(this, void 0, void 0, function* () {
        const reqUrl = `${fdConstants.baseUrl}${endPoint}`;
        let fetchOpts = {
            method: method,
            headers: {
                Authorization: fdAuth
            }
        };
        //@ts-ignore
        if (body && method === fdConstants.endpointMethodPost) {
            fetchOpts.body = JSON.stringify(body);
            fetchOpts.headers['Content-Type'] = 'application/json';
        }
        return yield processResponse(reqUrl, fetchOpts);
    });
}
function processResponse(reqUrl, fetchOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        let apiResp = yield fetch(reqUrl, fetchOpts);
        logger.info(`FD API: ${reqUrl} | ${fetchOpts.method} | Returned: ${apiResp.status}`);
        switch (apiResp.status) {
            case 201:
                return true;
                break;
            case 400:
                yield handleError(apiResp);
                break;
            case 404:
                yield handleError(apiResp);
                break;
            default:
                throw new exceptionalError(`Can't handle ${apiResp.status} right now!`);
        }
    });
}
function handleError(apiResp) {
    return __awaiter(this, void 0, void 0, function* () {
        let errString = `Request returned ${apiResp.status}`;
        if (apiResp.statusText) {
            errString = `${errString} ${apiResp.statusText}`;
        }
        logger.info(util.inspect(apiResp));
        let body = yield apiResp.text();
        if (body) {
            logger.info(body);
            let jsonBody;
            try {
                jsonBody = JSON.parse(body);
            }
            catch (_a) {
                errString = `${errString}\n${body}`;
            }
            if (jsonBody) {
                jsonBody = jsonBody;
                errString = `${errString}\n${jsonBody.description}`;
                if (jsonBody.errors) {
                    for (const err of jsonBody.errors) {
                        errString = `${errString}\n${err.field}: ${err.message}`;
                    }
                }
            }
        }
        throw new exceptionalError(errString);
    });
}
