var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import exceptionalError from 'exceptionalhandler';
import * as fdConstants from './constants.js';
import * as fdApi from './api.js';
import { valIsEmpty } from './email.preCanned.js';
import { existsSync, readFileSync } from 'fs';
export let templates = {};
export function loadTemplate(templateName, path) {
    if (!templateName || !path) {
        throw new exceptionalError('Gotta provide both templane name and path!');
    }
    if (!existsSync(path)) {
        throw new exceptionalError(`File not found @ ${path}`);
    }
    let html = readFileSync(path, 'utf8');
    if (!html) {
        throw new exceptionalError("File was empty!");
    }
    templates[templateName] = html;
}
export function generateTemplateEmail(templateName, emailVars) {
    if (!templateName || !emailVars) {
        throw new exceptionalError('Gotta provide both template name and emailVars!');
    }
    if (!templates.hasOwnProperty(templateName)) {
        throw new exceptionalError(`No template for ${templateName}`);
    }
    let emailStr = templates[templateName];
    for (let [emailVar, emailVal] of Object.entries(emailVars)) {
        emailVar = `{{${emailVar}}}`;
        if (!emailStr.includes(emailVar)) {
            throw new exceptionalError(`${emailVar} is not a valid arg for ${templateName}`);
        }
        if (typeof emailVal !== 'string') {
            throw new exceptionalError(`Non-string value provided for ${emailVar} in ${templateName}`);
        }
        const replaceRegex = new RegExp(emailVar, "gi");
        emailStr = emailStr.replace(replaceRegex, emailVal);
    }
    let endCheck = emailStr.match(/\{\{.+?\}\}/g);
    if (endCheck) {
        throw new exceptionalError(`Missing args ${endCheck.join(" | ")} for ${templateName}`);
    }
    return emailStr;
}
export function sendTemplateEmail(emailAddress, subject, templateName, templateVars) {
    return __awaiter(this, void 0, void 0, function* () {
        const roughEmailRegex = /^.+?@.+?\..+?$/i;
        if (!emailAddress || !emailAddress.match(roughEmailRegex)) {
            throw new exceptionalError(`Invalid Email`);
        }
        if (valIsEmpty(subject)) {
            throw new exceptionalError(`Invalid Subject`);
        }
        if (valIsEmpty(templateName) || !templates.hasOwnProperty(templateName)) {
            throw new exceptionalError(`Invalid Template Name`);
        }
        const emailHtml = generateTemplateEmail(templateName, templateVars);
        const newEmailPayload = {
            email: emailAddress,
            subject: subject,
            type: fdConstants.typeShipping,
            status: fdConstants.StatusClosed,
            priority: fdConstants.PriorityLow,
            description: emailHtml,
            email_config_id: fdConstants.mailboxBlueDriverMax,
            group_id: fdConstants.groupBlueDriver,
            tags: [fdConstants.tagShipping]
        };
        return yield fdApi.performApiReq(fdConstants.endpointSendEmail, fdConstants.endpointMethodPost, newEmailPayload);
    });
}
