// Important!
// For now this is *only* for sending emails
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import exceptionalError from 'exceptionalhandler';
import * as fdConstants from './freshdesk.constants.js';
import * as fdApi from './freshdesk.api.js';
export function sendShippingNotification(action, emailAddress, details) {
    return __awaiter(this, void 0, void 0, function* () {
        validateEmailFields(action, emailAddress, details);
        const emailPayload = generateShippingEmailPayload(action, emailAddress, details);
        return yield fdApi.performApiReq(fdConstants.endpointSendEmail, fdConstants.endpointMethodPost, emailPayload);
    });
}
function validateEmailFields(action, emailAddress, details) {
    if (valIsEmpty(emailAddress)) {
        throw new exceptionalError(`Email address is mandatory`);
    }
    if (![fdConstants.shippingActionOrdered, fdConstants.shippingActionShipped].includes(action)) {
        throw new exceptionalError(`Shipping action must be ordered or shipped`);
    }
    if (valIsEmpty(details.orderNumber)) {
        throw new exceptionalError(`Order number is mandatory`);
    }
}
function generateShippingEmailPayload(action, emailAddress, details) {
    const subject = `Shipping Update - Order ${details.orderNumber} ${action}`;
    let emailContents = [`Order: ${details.orderNumber}`];
    emailContents.push(`Status: ${action}`);
    const recipientKeys = {
        "name": "Name",
        "street": "Street",
        "city": "City",
        "stateProv": "State/Province",
        "zipPostal": "Zip/Postal Code",
        "country": "Country",
        "email": "Email"
    };
    for (const [recipientKey, displayVal] of Object.entries(recipientKeys)) {
        //@ts-ignore
        let keyVal = details.recipient[recipientKey];
        if (!valIsEmpty(keyVal)) {
            emailContents.push(`${displayVal}: ${keyVal}`);
        }
    }
    for (const [itemName, qty] of Object.entries(details.items)) {
        if (!valIsEmpty(itemName) && !valIsEmpty(qty)) {
            emailContents.push(`${itemName}: ${qty}`);
        }
    }
    if (Object.keys(details.optionalFields).length > 0) {
        for (const [fieldName, fieldVal] of Object.entries(details.optionalFields)) {
            if (!valIsEmpty(fieldName) && !valIsEmpty(fieldVal)) {
                emailContents.push(`${fieldName}: ${fieldVal}`);
            }
        }
    }
    let newEmailPayload = {
        email: emailAddress,
        subject: subject,
        type: fdConstants.typeShipping,
        status: fdConstants.StatusClosed,
        priority: fdConstants.PriorityLow,
        description: emailContents.join("\r\n<br>"),
        email_config_id: fdConstants.mailboxBlueDriverMax,
        group_id: fdConstants.groupBlueDriver,
        tags: [fdConstants.tagShipping]
    };
    return newEmailPayload;
}
function valIsEmpty(val) {
    return (!val || val === "");
}
