var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as freshdesk from '../freshdesk.js';
import fetchMock from 'fetch-mock';
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
const expect = chai.expect;
chai.use(chaiAsPromised);
import * as dotenv from 'dotenv';
dotenv.config();
const isProduction = (process.env.NODE_ENV === 'PRODUCTION');
describe('Testing the freshdesk guy', () => __awaiter(void 0, void 0, void 0, function* () {
    let postMock;
    let testRecipient = {
        orderNumber: "Order123456",
        recipient: {
            email: "someDude@domain.com",
            name: "Some Dude",
            street: "123 Street Pl",
            city: "New York",
            stateProv: "CA",
            zipPostal: "123456",
            country: "US"
        },
        items: {
            "aThing": 4,
            "anotherThing": 5
        },
        optionalFields: {
            "FieldOne": "ValOne",
            "FieldTwo": "Hello"
        }
    };
    context('It sends the shipping email', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            if (!isProduction) {
                fetchMock.reset();
                postMock = fetchMock.post(/./, (url, opts) => {
                    return verifyPostReq(url, opts);
                });
            }
        }));
        it('Validates', () => __awaiter(void 0, void 0, void 0, function* () {
            let testVal = {};
            yield expect(freshdesk.sendShippingNotification("blah", "", testVal)).to.be.rejectedWith("Email address is mandatory");
            yield expect(freshdesk.sendShippingNotification("blah", "asdasd@asdsad.com", testVal)).to.be.rejectedWith("Shipping action must be ordered or shipped");
            yield expect(freshdesk.sendShippingNotification("Ordered", "asdasd@asdsad.com", testVal)).to.be.rejectedWith("Order number is mandatory");
        }));
        it('Works', () => __awaiter(void 0, void 0, void 0, function* () {
            let sendEmail = yield freshdesk.sendShippingNotification("Ordered", "cnevin@yert.ca", testRecipient);
            expect(sendEmail).to.be.true;
            expect(postMock.calls().length).to.equal(1);
        }));
        it('Parses Error Messages', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(freshdesk.sendShippingNotification("Ordered", "this@fail.com", testRecipient)).to.be.rejectedWith("someField: Error Message");
            yield expect(freshdesk.sendShippingNotification("Ordered", "this@fail.com", testRecipient)).to.be.rejectedWith("someOtherField: Some Other Error Message");
        }));
    }));
}));
function verifyPostReq(url, opts) {
    const expectedKey = `Basic c29tZUtleVdoYXRldmVySWR1bm5vITpY`;
    if (!opts.headers.Authorization || opts.headers.Authorization !== expectedKey) {
        throw `BAD AUTH`;
    }
    let jsonBody = JSON.parse(opts.body);
    if (jsonBody.email === "this@fail.com") {
        return {
            status: 400,
            body: {
                "description": "Validation failed",
                "errors": [
                    {
                        "field": "someField",
                        "message": "Error Message",
                        "code": "invalid_value"
                    },
                    {
                        "field": "someOtherField",
                        "message": "Some Other Error Message",
                        "code": "invalid_value"
                    }
                ]
            }
        };
    }
    let expectedBody = {
        email: 'cnevin@yert.ca',
        subject: 'Shipping Update - Order Order123456 Ordered',
        type: 'Shipping &amp; Ordering',
        status: 5,
        priority: 1,
        description: 'Order: Order123456\r\n<br>Status: Ordered\r\n<br>Name: Some Dude\r\n<br>Street: 123 Street Pl\r\n<br>City: New York\r\n<br>State/Province: CA\r\n<br>Zip/Postal Code: 123456\r\n<br>Country: US\r\n<br>Email: someDude@domain.com\r\n<br>aThing: 4\r\n<br>anotherThing: 5\r\n<br>FieldOne: ValOne\r\n<br>FieldTwo: Hello',
        email_config_id: 43000161966,
        group_id: 43000612165,
        tags: ['Shipping Notification']
    };
    if (JSON.stringify(expectedBody) !== opts.body) {
        const stringComp = compareStrings(JSON.stringify(expectedBody), opts.body);
        console.log(`Match failure - expected:\n${stringComp.str1}\n\nbut got: \n${stringComp.str2}`);
        //console.log(util.inspect(opts.body))
        throw `WROOONG`;
    }
    return 201;
}
function compareStrings(str1, str2) {
    let str1Arr = str1.match(/./g);
    let str2Arr = str2.match(/./g);
    let longestArr = [str1Arr.length, str2Arr.length].sort(function (a, b) { return a - b; })[1];
    let printSt1 = "";
    let printSt2 = "";
    for (let i = 0; i < longestArr; i++) {
        if (str1Arr[i] === str2Arr[i]) {
            printSt1 = `${printSt1}${str1Arr[i]}`;
            printSt2 = `${printSt2}${str1Arr[i]}`;
        }
        else {
            if (str1Arr[i] === undefined) {
                printSt1 = `${printSt1} `;
                printSt2 = `${printSt2}${printColor(str2Arr[i], "red")}`;
            }
            else if (str2Arr[i] === undefined) {
                printSt1 = `${printSt1}${printColor(str1Arr[i], "red")}`;
                printSt2 = `${printSt2} `;
            }
            else {
                printSt1 = `${printSt1}${printColor(str1Arr[i], "red")}`;
                printSt2 = `${printSt2}${printColor(str2Arr[i], "red")}`;
            }
        }
    }
    return { str1: printSt1, str2: printSt2 };
}
function printColor(input, color) {
    switch (color) {
        case "red":
            return `\x1b[91m${input}\x1b[0m`;
            break;
        case "blue":
            return `\x1b[94m${input}\x1b[0m`;
            break;
        default:
            throw `unknown color ${color}`;
    }
}
