var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
const expect = chai.expect;
chai.use(chaiAsPromised);
import * as dotenv from 'dotenv';
dotenv.config();
const isProduction = (process.env.NODE_ENV === 'PRODUCTION');
import * as templateEmail from '../email.template.js';
describe('Testing email template system', () => __awaiter(void 0, void 0, void 0, function* () {
    context('Handlng the template', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Detects bad paths/names or empty files', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.loadTemplate(null, "asdadasdsa.htm"); }).to.throw('Gotta provide both templane name and path!');
            expect(() => { templateEmail.loadTemplate("test", ""); }).to.throw('Gotta provide both templane name and path!');
            expect(() => { templateEmail.loadTemplate("test", "asdadasdsa.htm"); }).to.throw('File not found @ asdadasdsa.htm');
        }));
        it('Catches empty templates', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.loadTemplate("test", "src/tests/html/empty.html"); }).to.throw("File was empty!");
        }));
        it('Loads the template', () => __awaiter(void 0, void 0, void 0, function* () {
            let expectedHtml = '<body>\r\n' +
                '    <h1>HELLO FRIEND</h1>\r\n' +
                '    <div>A value will go here: <b>{{VAL1}}</b> and here:<b>{{OTHERVAL}}</b></div>\r\n' +
                '</body>';
            templateEmail.loadTemplate("test", "src/tests/html/test.html");
            expect(templateEmail.templates).to.haveOwnProperty("test");
            expect(templateEmail.templates.test).to.equal(expectedHtml);
        }));
    }));
    context('Handlng the template', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Handles bad args', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.generateTemplateEmail(null, {}); }).to.throw('Gotta provide both template name and emailVars!');
            expect(() => { templateEmail.generateTemplateEmail("", {}); }).to.throw('Gotta provide both template name and emailVars!');
            expect(() => { templateEmail.generateTemplateEmail("asdada", null); }).to.throw('Gotta provide both template name and emailVars!');
        }));
        it('Catches non-existant templates', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.generateTemplateEmail("asdaf", {}); }).to.throw("No template for asdaf");
        }));
        it('Catches missing args', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.generateTemplateEmail("test", {}); }).to.throw("Missing args {{VAL1}} | {{OTHERVAL}} for test");
        }));
        it('Catches extra args', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.generateTemplateEmail("test", { "VAL1": "someText", "someInvalidArg": "sadad" }); }).to.throw("{{someInvalidArg}} is not a valid arg for test");
        }));
        it('Catches non-string args', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { templateEmail.generateTemplateEmail("test", { "VAL1": 4567987 }); }).to.throw("Non-string value provided for {{VAL1}} in test");
        }));
        let goodHtml = '<body>\r\n' +
            '    <h1>HELLO FRIEND</h1>\r\n' +
            '    <div>A value will go here: <b>asdad</b> and here:<b>sadhdqhdiuwhde!!!</b></div>\r\n' +
            '</body>';
        it('Works!', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(templateEmail.generateTemplateEmail("test", { "VAL1": "asdad", "OTHERVAL": "sadhdqhdiuwhde!!!" })).to.equal(goodHtml);
        }));
        let goodHtml2 = '<body>\r\n' +
            '    <h1>HELLO FRIEND</h1>\r\n' +
            '    <div>A value will go here: <b>qqqqqqqq</b> and here:<b>sdasdqwewq!?!#!</b> ALSO here:<b>sdasdqwewq!?!#!</b></div>\r\n' +
            '</body>';
        templateEmail.loadTemplate("test2", "src/tests/html/testDuplicateFields.html");
        it('Works With Two Args!', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(templateEmail.generateTemplateEmail("test2", { "VAL1": "qqqqqqqq", "OTHERVAL": "sdasdqwewq!?!#!" })).to.equal(goodHtml2);
        }));
    }));
}));
